"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
var Container = /** @class */ (function () {
    function Container(parentContainer) {
        this._map = new Map();
        this._parentContainer = parentContainer;
    }
    Container.prototype.get = function (key) {
        var isMultiInject = Array.isArray(key);
        if (Array.isArray(key)) {
            if (key.length > 0) {
                key = key[0];
            }
            else {
                throw new Error("Invalid multi inject of `" + key.toString() + "`.");
            }
        }
        var entries = this._map.get(key);
        if (isMultiInject && (!entries || entries.length === 0)) {
            return [];
        }
        if (!entries || entries.length === 0) {
            if (this._parentContainer) {
                return this._parentContainer.get(key);
            }
            throw new Error("Missing binding of `" + key.toString() + "`.");
        }
        var resolvedEntries = [];
        for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
            var entry = entries_1[_i];
            if (!entry.value && entry.kind === 'newable') {
                var proxyObj = {
                    self: this,
                    entry: entry,
                    getValue: function () { }
                };
                proxyObj.getValue = function (obj) {
                    if (obj.entry.kind === 'newable' && !obj.entry.value) {
                        obj.entry.value = obj.self.resolve(obj.entry.newable, obj.entry.params);
                    }
                    return obj.entry.value;
                }.bind(null, proxyObj);
                var proxy = new Proxy(proxyObj, {
                    get: function (obj, prop) {
                        var entryValue = obj.getValue();
                        return entryValue[prop];
                    },
                    set: function (obj, prop, value) {
                        var entryValue = obj.getValue();
                        entryValue[prop] = value;
                        return true;
                    },
                    deleteProperty: function (obj, prop) {
                        var entryValue = obj.getValue();
                        delete entryValue[prop];
                        return true;
                    },
                    enumerate: function (obj) {
                        var entryValue = obj.getValue();
                        return Object.keys(entryValue);
                    },
                    has: function (obj, key) {
                        var entryValue = obj.getValue();
                        return key in entryValue;
                    },
                    setPrototypeOf: function (obj, proto) {
                        var entryValue = obj.getValue();
                        Object.setPrototypeOf(entryValue, proto);
                        return true;
                    },
                    preventExtensions: function (obj) {
                        Object.preventExtensions(obj.getValue());
                        return true;
                    },
                    ownKeys: function (obj) {
                        return Object.keys(obj.getValue());
                    },
                    isExtensible: function (obj) {
                        return Object.isExtensible(obj.getValue());
                    },
                    getPrototypeOf: function (obj) {
                        return Object.getPrototypeOf(obj.getValue());
                    },
                    getOwnPropertyDescriptor: function (obj, key) {
                        return Object.getOwnPropertyDescriptor(obj.getValue(), key);
                    },
                    defineProperty: function (obj, key, desc) {
                        Object.defineProperty(obj.getValue(), key, desc);
                        return true;
                    },
                    construct: function (obj, args) {
                        var value = obj.getValue();
                        return new (value.bind.apply(value, __spreadArrays([void 0], args)))();
                    },
                    apply: function (obj, self, args) {
                        return obj.getValue().apply(self, args);
                    }
                });
                if (isMultiInject) {
                    resolvedEntries.push(proxy);
                }
                else {
                    return proxy;
                }
            }
            else {
                if (entry.kind === 'factory' && !entry.value) {
                    entry.value = entry.factory(this);
                }
                if (isMultiInject) {
                    resolvedEntries.push(entry.value);
                }
                else {
                    return entry.value;
                }
            }
        }
        return resolvedEntries;
    };
    Container.prototype.bind = function (serviceIdentifier, constructor, params) {
        if (params === void 0) { params = []; }
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'newable',
            newable: constructor,
            params: params
        });
        this._map.set(serviceIdentifier, entries);
    };
    Container.prototype.bindToConstant = function (serviceIdentifier, constant) {
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'constant',
            value: constant
        });
        this._map.set(serviceIdentifier, entries);
    };
    /**
     * Bind the implementations of the class to the class.
     * @param constructor
     * @param params
     * @param serviceIdentifiers
     */
    Container.prototype.bindToImplements = function (constructor, params, serviceIdentifiers) {
        if (params === void 0) { params = []; }
        if (serviceIdentifiers === void 0) { serviceIdentifiers = []; }
        var entry = {
            kind: 'newable',
            newable: constructor,
            params: params
        };
        for (var _i = 0, serviceIdentifiers_1 = serviceIdentifiers; _i < serviceIdentifiers_1.length; _i++) {
            var serviceIdentifier = serviceIdentifiers_1[_i];
            var entries = this._map.get(serviceIdentifier) || [];
            entries.push(entry);
            this._map.set(serviceIdentifier, entries);
        }
    };
    Container.prototype.bindToFactory = function (serviceIdentifier, factory) {
        var entries = this._map.get(serviceIdentifier) || [];
        entries.push({
            kind: 'factory',
            factory: factory
        });
        this._map.set(serviceIdentifier, entries);
    };
    Container.prototype.resolve = function (service, params) {
        if (params === void 0) { params = []; }
        var args = [];
        for (var _i = 0, params_1 = params; _i < params_1.length; _i++) {
            var val = params_1[_i];
            args.push(this.get(val));
        }
        return new (service.bind.apply(service, __spreadArrays([void 0], args)))();
    };
    Container.prototype.resolveFactory = function (service, params) {
        if (params === void 0) { params = []; }
        var args = [];
        for (var _i = 0, params_2 = params; _i < params_2.length; _i++) {
            var val = params_2[_i];
            args.push(this.get(val));
        }
        return service.bind.apply(service, __spreadArrays([undefined], args));
    };
    return Container;
}());
exports.Container = Container;
//# sourceMappingURL=api.js.map